package org.bolson.vote;
import java.util.Vector;

/**
* Raw summation of voter's ratings.
 * External limits must be established.
 */
public class IRNR extends RatedVotingSystem {
	Vector votes = new Vector();
	int voterToTrack = -1;
	boolean showIntermedite = false;
    public IRNR( int numCandidates ) {
    	super( numCandidates );
		talley = new double[numc];
    }

	public void init( String argv[] ) {
		// default impl, do nothing
		for ( int i = 0; i < argv.length; i++ ) {
			if ( argv[i].equals( "track" ) ) {
				i++;
				voterToTrack = Integer.parseInt( argv[i] );
			} else if ( argv[i].equals( "show" ) ) {
				showIntermedite = true;
//			} else if ( argv[i].equals( "" ) ) {
			} else {
				System.err.println( "IRNR.init: bogus arg \"" + argv[i] + '"' );
				return;
			}
		}
	}

    public int voteRating( int rating[] )  {
		votes.add( rating.clone() );
		int i;
		return 0;
    }
    public int voteRating( float rating[] )  {
		votes.add( rating.clone() );
		return 0;
    }
    public int voteRating( double rating[] )  {
		votes.add( rating.clone() );
		return 0;
    }
    
    public int[] getWinners() {
		double max = Double.MIN_VALUE;
		int i;
		int numWinners = 1;
		int numActive = numc;
		boolean active[] = new boolean[numc];
		int choiceIndecies[] = new int[numc];
		double dt[] = new double[numc];
		for ( int c = 0; c < numc; c++ ) {
			active[c] = true;
		}
		while ( numActive > 1 ) {
			// per IR setup
			{
				int curact = 0;
				for ( int c = 0; c < numc; c++ ) {
					talley[c] = 0;
					if ( active[c] ) {
						choiceIndecies[curact] = c;
						curact++;
					}
				}
			}
			// sum up into talley
			for ( int v = 0; v < votes.size(); v++ ) {
				Object o;
				double ts = 0.0;
				o = votes.elementAt( v );
				if ( o instanceof int[] ) {
					int[] ot = (int[])o;
					for ( int c = 0; c < numc; c++ ) if ( ot[c] != NO_VOTE && active[c] ) {
						//System.out.println("int[] ot[" + c + "] = " + ot[c] );
						ts += Math.abs(ot[c]);
					}
					for ( int c = 0; c < numc; c++ ) if ( ot[c] != NO_VOTE && active[c] ) {
						talley[c] += ot[c] / ts;
					}
				} else if ( o instanceof float[] ) {
					float[] ot = (float[])o;
					for ( int c = 0; c < numc; c++ ) if ( active[c] ) {
						//System.out.println("float[] ot[" + c + "] = " + ot[c] );
						ts += Math.abs(ot[c]);
					}
					for ( int c = 0; c < numc; c++ ) if ( active[c] ) {
						//System.out.println("float[] ot[" + c + "] = " + ot[c] + " / (ts = " + ts + ") = " + (ot[c] / ts) );
						talley[c] += ot[c] / ts;
					}
					if ( v == voterToTrack ) {
						for ( int c = 0; c < numc; c++ ) {
							if ( active[c] ) {
								System.out.print( ot[c] + " / " + ts + " = " + (ot[c] / ts) + '\t' );
							} else {
								System.out.print( ot[c] + " disabled = 0\t" );
							}
						}
						System.out.println();
					}
				} else if ( o instanceof double[] ) {
					double[] ot = (double[])o;
					for ( int c = 0; c < numc; c++ ) if ( active[c] ) {
						//System.out.println("double[] ot[" + c + "] = " + ot[c] );
						ts += Math.abs(ot[c]);
					}
					for ( int c = 0; c < numc; c++ ) if ( active[c] ) {
						talley[c] += ot[c] / ts;
					}
				} else {
					System.err.println( "bogus vote \"" + o + '\"' );
				}
			}
			// sort
			boolean notdone = true;
			while ( notdone ) {
				notdone = false;
				for ( int c = 1; c < numActive; c++ ) {
					if ( talley[choiceIndecies[c]] > talley[choiceIndecies[c-1]] ) {
						int ti = choiceIndecies[c];
						choiceIndecies[c] = choiceIndecies[c-1];
						choiceIndecies[c-1] = ti;
						notdone = true;
					}
				}
			}
			if ( showIntermedite ) {
				System.out.println("num active " + numActive );
				for ( int c = 0; c < numc; c++ ) {
					int tci;
					tci = choiceIndecies[c];
					System.out.println("\tchoiceIndecies[" + c + "] = " + tci +
									   "\ttalley[" + tci + "] = " + talley[tci] +
									   "\tactive[" + tci + "] = " + active[tci] );
				}
			}
			if ( talley[choiceIndecies[0]] == talley[choiceIndecies[numActive-1]] ) {
				// N-way tie.
				numWinners = numActive;
				break;
			}
			active[choiceIndecies[numActive-1]] = false;
			numActive--;
			while ( (numActive > 1) &&
					(talley[choiceIndecies[numActive-1]] == talley[choiceIndecies[numActive]]) ) {
				// eliminate all who tied for last
				active[choiceIndecies[numActive-1]] = false;
				numActive--;
			}
		}
		int winners[] = new int[numWinners];
		for ( i = 0; i < numWinners; i++ ) {
			winners[i] = choiceIndecies[i];
		}
		return winners;
    }
	
    
    void squashToAcceptance( int rating[] ) {
		for ( int i = 0; i < numc; i++ ) {
			if ( rating[i] >= 0 ) {
				rating[i] = 1;
			} else {
				rating[i] = 0;
			}
		}
    }
    
    protected double talley[];
    
    public String toString() {
		StringBuffer sb = new StringBuffer();
		for ( int i = 0; i < talley.length; i++ ) {
			sb.append( talley[i] ).append('\n');
		}
		return sb.toString();
    }
	
    public String toString( String names[] ) {
		StringBuffer sb = new StringBuffer();
		for ( int i = 0; i < talley.length; i++ ) {
			sb.append( names[i] ).append('\t');
			sb.append( talley[i] ).append('\n');
		}
		return sb.toString();
    }
	
    public String htmlSummary( String names[] ) {
		StringBuffer sb;
		double max;
		boolean in[] = new boolean[numc];
		int maxi = 0,i;
		for ( i = 0; i < numc; i++ ) {
			in[i] = true;
		}
		if ( names != null ) {
			sb = new StringBuffer( "<table><tr><th></th><th>Rating Summation</th></tr>\n" );
		} else {
			sb = new StringBuffer( "<table><tr><th>Choice Index</th><th>Rating Summation</th></tr>\n" );
		}
		while ( true ) {
			boolean done;
			max = Double.MIN_VALUE;
			done = true;
			for ( i = 0; i < numc; i++ ) {
				if ( in[i] && talley[i] > max ) {
					done = false;
					maxi = i;
					max = talley[i];
				}
			}
			if ( done ) break;
			i = maxi;
			in[i] = false;
			sb.append( "<tr><td>" );
			if ( names != null ) {
				sb.append( names[i] );
			} else {
				sb.append( i+1 );
			}
			sb.append( "</td><td>" );
			sb.append( talley[i] );
			sb.append( "</td></tr>\n" );
		}
		sb.append( "</table>\n" );
		return sb.toString();
    }
};
